/*

This source file is part of the Solar Visitor program, (c) 2007
Graeme Sheppard. Released under the GPLv2.

*/

#ifndef SolarListener
#define SolarListener

#include "Ogre.h"
#include "OgreKeyEvent.h"
#include "OgreEventListeners.h"

#include "Types.h"
#include "SolarSystem.h"
#include "Menus.h"

using namespace Ogre;
 
class Listener : public FrameListener, public KeyListener,
  MouseMotionListener, public MouseListener
{
  public:
    Listener (RenderWindow*, Camera*, SceneManager*, int);
    ~Listener();

    bool frameStarted (const FrameEvent &evt);
    bool frameEnded (const FrameEvent &evt);

    void mouseMoved (MouseEvent *evt);
    void mouseDragged (MouseEvent*) {}
    void mouseClicked (MouseEvent*) {}
    void mouseEntered (MouseEvent*) {}
    void mouseExited (MouseEvent*) {}
    void mouseReleased (MouseEvent*) {}
    void mousePressed (MouseEvent*) {}

    void keyPressed (KeyEvent *evt);
    void keyReleased (KeyEvent *evt);
    void keyClicked (KeyEvent*) {}

    void setCommandText (const char *szText, const char *szTabName = NULL);
    bool cmpCommandText (const char *szText);
    void showDebugText (const char *szText);

    void moveKey (int iKey, Real iDisplacement, bool bShift);
    void numPress (int iKey);
    void showTabMenu (const char *szMenuName);    
    void resetView();
    void plotPlanet (Planet *oPlanet, sv64 iTime);
    bool cmpCommand (const char *szAction);

    void updateStats();

    RenderWindow *oRenderWindow;
    SceneManager *oSceneMgr;
    SceneNode *oCameraNode;
    SceneNode *oCameraYawNode;
    SceneNode *oCameraPitchNode;
    SceneNode *oCameraRollNode;
    Camera *oCamera;
    EventProcessor *oEventProcessor;

    Overlay *oSolarOverlay;
    Overlay *oOgreOverlay;

    SolarSystem *oSolarSystem;
    Menus oMenu;

    int iModeCode;
    int iReturnCode;
    bool bContinue;
    bool bStatsOn;
    bool bSimpleFPS;
    bool bShiftPressed;
    bool bMobileLight;
    Real rRotate;
    Real rMove;
    Vector3 oDirection;
    sv64 iSolarVisitorTime; // time in milliseconds
    sv64 iSolarVisitorTimeLast; // so time checks don't need to be performed
    sv64 iTimeWarp; // the adjust to iSVT every second of user time
    sv64 iSolarTimeOrigin;
    int iUserTimeOrigin; // when a key pressed
    int iTimeKeysReleased;
    int iTimeLastDebugMsg;
    String sCommandText;
    String sTabMenu;
    String sScreenShotFileName;
    int iScreenShotCount;
    int iNumUserLights;
    static const int iMaxUserLights = 20;
    const char *szTextCommand;
};

#endif
