/*

This source file is part of the Solar Visitor program, (c) 2007
Graeme Sheppard. Released under the GPLv2.

*/


#include "Ogre.h"

using namespace Ogre;

#if OGRE_PLATFORM == PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"

// Ogre's getMilliseconds doesn't work under wine, do it manually

int getUserTime()
{
  static int iOrigin = -1;
  int iNow;
  
  iNow = GetTickCount();
  if (iOrigin != -1)
    return iNow - iOrigin;
  else
  {
    iOrigin = iNow;
    return 0;
  }
}

void setForeground()
{
// windows makes even the simplest tasks difficult
  DWORD dwCurrent, dwSolar;
  HWND hwSolar;

  dwCurrent = GetWindowThreadProcessId (GetForegroundWindow(), NULL);
  dwSolar = GetWindowThreadProcessId (GetActiveWindow(), NULL);
  hwSolar = GetActiveWindow();
  if (!dwCurrent || !dwSolar || !dwSolar)
    return;
  if (dwCurrent == dwSolar)
    return;
  if (!AttachThreadInput (dwCurrent, dwSolar, TRUE))
    return;
  SetForegroundWindow (hwSolar);
  AttachThreadInput (dwCurrent, dwSolar, FALSE);
  BringWindowToTop (hwSolar);
}

#else

int getUserTime()
{
  static Ogre::Timer *oTimer = NULL;
  
  if (oTimer != NULL)
    return oTimer->getMilliseconds();
  else
  {
    oTimer = new Ogre::Timer();
    oTimer->reset();
    return 0;
  }
}

void setForeground()
{
// linux is pretty sane
}

#endif
 
