/*

This source file is part of the Solar Visitor program, (c) 2007
Graeme Sheppard. Released under the GPLv2.

*/

#include <stdlib.h>

#include "SVmain.h"
#include "Listener.h"
#include "Platform.h"

using namespace Ogre;


SVmain::~SVmain()
{
  if (oListener)
    delete oListener;
  if (oRoot)
    delete oRoot;
}


int SVmain::run (int iModeCode)
{
  Viewport* vp;
  String sCfgPath;
  int iReturnCode;

  oListener = 0;
  oRoot = 0;
  sCfgPath = "../cfg/";

// third arguement was ogre.log
  oRoot = new Root (sCfgPath + "plugins.cfg", sCfgPath + "ogre.cfg", "");
  setupResources (sCfgPath);
  if (iModeCode == sv_start_screen_mode || !oRoot->restoreConfig())
    if (!oRoot->showConfigDialog()) return sv_exit_normal;
  oWindow = oRoot->initialise (true);
  oSceneMgr = oRoot->createSceneManager (ST_GENERIC, "ExampleSMInstance");
  oCamera = oSceneMgr->createCamera ("PlayerCam"); 
  oCamera->setNearClipDistance (5);
  vp = oWindow->addViewport (oCamera);
  vp->setBackgroundColour (ColourValue (0, 0, 0));
  setForeground();
  oCamera->setAspectRatio (
    Real (vp->getActualWidth()) / Real (vp->getActualHeight()));
  TextureManager::getSingleton().setDefaultNumMipmaps (5);
  ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
  oSceneMgr->setShadowTechnique (SHADOWTYPE_STENCIL_ADDITIVE);
  oSceneMgr->setAmbientLight (ColourValue (0.35, 0.35, 0.35));
  oListener = new Listener (oWindow, oCamera, oSceneMgr, iModeCode);
  oRoot->addFrameListener (oListener);
  oRoot->startRendering();
  iReturnCode = oListener->iReturnCode;
  delete oListener;
  delete oRoot;
  oListener = 0;
  oRoot = 0;
  return iReturnCode;
}


void SVmain::setupResources (String szCfgPath)
{
  String sName, sType, sArch;
  ConfigFile cf;
  ConfigFile::SettingsMultiMap *settings;
  ConfigFile::SettingsMultiMap::iterator i;


  cf.load (szCfgPath + "resources.cfg");
  ConfigFile::SectionIterator si = cf.getSectionIterator();

  while (si.hasMoreElements())
  {
    sName = si.peekNextKey();
    settings = si.getNext();
    for (i = settings->begin(); i != settings->end(); ++i)
    {
      sType = i->first;
      sArch = i->second;
      ResourceGroupManager::getSingleton().addResourceLocation (
        sArch, sType, sName);
     }
  }
}

 
#if OGRE_PLATFORM == PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"

INT WINAPI WinMain (HINSTANCE hInst, HINSTANCE, LPSTR szArg, INT)
{
  int iRet;

  try
  {
    SVmain app;

    iRet = app.run (atoi (szArg));
  }
  catch (Exception &e)
  {
    MessageBox (NULL, e.getFullDescription().c_str(),
      "An exception has occured!", MB_OK | MB_ICONERROR | MB_TASKMODAL);
    iRet = sv_exit_exception;
  }
//  ExitProcess (iRet);
  return iRet;
}

#else

int main (int argc, char **argv)
{
  int iRet;

  try
  {
    SVmain app;

    iRet = app.run (argc == 2 ? atoi (argv [1]) : 0);
  }
  catch (Exception &e)
  {
    fprintf (stderr, "An exception has occured: %s\n",
      e.getFullDescription().c_str());
    iRet = sv_exit_exception;
  }
  return iRet;
}

#endif
